import openpyxl as opxl
from openpyxl.utils import column_index_from_string
import re
import argparse
from itertools import chain
from pathlib import Path
import logging as lgg

number_group = r'\s*(-?\d+\.?\d*)'
number = number_group.replace('(', '').replace(')', '')
energies = re.compile(
    r' Zero-point correction=\s*(-?\d+\.?\d*) \(Hartree/Particle\)\n'
    r' Thermal correction to Energy=\s*(-?\d+\.?\d*)\n'
    r' Thermal correction to Enthalpy=\s*(-?\d+\.?\d*)\n'
    r' Thermal correction to Gibbs Free Energy=\s*(-?\d+\.?\d*)\n'
    r' Sum of electronic and zero-point Energies=\s*(-?\d+\.?\d*)\n'
    r' Sum of electronic and thermal Energies=\s*(-?\d+\.?\d*)\n'
    r' Sum of electronic and thermal Enthalpies=\s*(-?\d+\.?\d*)\n'
    r' Sum of electronic and thermal Free Energies=\s*(-?\d+\.?\d*)'
)  # use .search(text).groups()
frequencies = re.compile(
    'Frequencies(?:\s+(?:Fr= \d+)?--\s+)' + 3 * number_group
)


def get_args(argv=None):
    prs = argparse.ArgumentParser(
        description='Process gaussian optimization files to get energies and '
                    'number of imaginary frequencies.'
    )
    prs.add_argument(
        'files', type=Path, nargs='+',
        help='One or more gaussian output files or directories with such.'
    )
    prs.add_argument(
        '-f', '--file', type=Path, default=None,
        help='Wries output to specified excel file.'
    )
    prs.add_argument(
        '-c', '--corrections', action='store_true',
        help='Include corrections in the output.'
    )
    prs.add_argument(
        '-i', '--imag_count', action='store_true',
        help='Include number of imaginary numbers in the output.'
    )
    prs.add_argument(
        '-e', '--energies', action='store_true',
        help='Include energies values in the output.'
    )
    prs.add_argument(
        '-a', '--append_entry', metavar='COL',
        help='Append output values to entry, if it already exists, otherwise '
             'add a new entry. Values will be written to sheet starting at '
             'column COL. Column letter or 1-based numerical index may be '
             'given.'
    )
    log = prs.add_mutually_exclusive_group()
    log.add_argument(
        '-u', '--unconverged', action='store_true',
        help='Print names of files that did not converged.'
    )
    log.add_argument(
        '-s', '--silent', action='store_true',
        help='Suppress printing to sdtout.'
    )
    args = prs.parse_args(argv)
    if args.append_entry is not None and args.file is None:
        prs.error("--append_entry needs --file to be specified.")
    return args


def get_data(path):
    with path.open('r') as file:
        text = file.read()
    ens = energies.search(text)
    freqs = frequencies.findall(text)
    if not ens or not freqs:
        lgg.debug(f'NOT CONVERGED: {path.name}')
        return []
    else:
        imag = sum(float(f) < 0 for line in freqs for f in line)
        return [*ens.groups(), imag]


def select_data(line, args):
    entries = [bool(line)]
    # first element indicates if calculation converged
    if not line:
        return entries
    zcr, tcr, ecr, gcr, zpe, ten, ent, gib, imag = line
    # entry: name, value
    if args.energies:
        entries.append(("ZPE", float(zpe)))
        entries.append(("TEN", float(ten)))
        entries.append(("ENT", float(ent)))
        entries.append(("GIB", float(gib)))
    if args.corrections:
        entries.append(("ZPECORR", float(zcr)))
        entries.append(("TENCORR", float(tcr)))
        entries.append(("ENTCORR", float(ecr)))
        entries.append(("GIBCORR", float(gcr)))
    if args.imag_count:
        entries.append(("Imag.Freqs", int(imag)))
    return entries


def main(argv=None):
    args = get_args(argv)
    lgg.basicConfig(level='WARNING' if args.silent else 'INFO')
    dirs = (path for path in args.files if path.is_dir())
    inner_files = (
        path for dir in dirs for path in dir.iterdir()
        if path.is_file() and path.suffix in ['.log', '.out']
    )
    args_files = (
        path for path in args.files
        if path.is_file() and path.suffix in ['.log', '.out']
    )
    files = chain(args_files, inner_files)
    lines = ((path.name, get_data(path)) for path in files)
    data = ((file, select_data(line, args)) for file, line in lines)
    if args.file:
        wb = opxl.load_workbook(str(args.file))
        sheet = wb.active
        owned = {row[0].value: row[0].row for row in sheet.iter_rows(max_col=1)}
    else:
        wb, sheet, owned = None, None, {}
    unconverged = []
    length = 0
    if args.append_entry is not None:
        try:
            args.append_entry = int(args.append_entry)
        except ValueError:
            args.append_entry = column_index_from_string(args.append_entry)
    for file, (converged, *entries) in data:
        if not converged:  # no data extracted
            unconverged.append(file)
            continue
        if not entries:
            # only unconverged requested
            continue
        length = len(file) if len(file) > length else length
        lgg.info(
            ' '.join((
                f"{file: <{length}} -",
                *("{}= {}".format(*e) for e in entries)
            ))
        )
        if args.file and args.append_entry is not None:
            if file in owned:
                row = owned[file]
            else:
                row = sheet.max_row + 1
                sheet.cell(row=row, column=1, value=file)
            col = args.append_entry
            for n, value in enumerate(entries):
                sheet.cell(row=row, column=col+n, value=value[1])
        elif args.file:
            sheet.append([
                file, '',  # place for comment on this file
                *(e[1] for e in entries)
            ])
    if args.unconverged:
        lgg.info(f"{len(unconverged)} unconverged files:")
        for filename in unconverged:
            lgg.info(f'\t{filename}')
    if args.file:
        wb.save(str(args.file))


if __name__ == '__main__':

    main()



import argparse
import logging as lgg
from itertools import chain
from pathlib import Path


def fix_cont(mol: str, version: str = 'V2000') -> str:
    mol = mol.replace('    0\n', ' ' + version + '\n', 1)
    if not mol.endswith("M END\n"):
        mol = mol + "M END\n"
    return mol


def fix_file(file: Path, version: str = 'V2000'):
    with file.open('r+') as molfile:
        cont = molfile.read()
        if cont.endswith("M END\n") or '    0\n' not in cont:
            lgg.info(f"File {file.name} already looks good.")
        else:
            new = fix_cont(cont, version)
            molfile.seek(0)
            molfile.write(new)


def get_args(argv=None):
    prs = argparse.ArgumentParser()
    prs.add_argument(
        'files', type=Path, nargs='+',
        help='One or more .mol files created by Gaussview or directories '
             'containing such files.'
    )
    prs.add_argument(
        '-v', type=str, default='V2000',
        help='.mol file version, default is V2000'
    )
    prs.add_argument(
        '-s', '--silent', action='store_true',
        help='suppress printing to sdtout'
    )
    return prs.parse_args(argv)


def main(argv=None):
    args = get_args(argv)
    lgg.basicConfig(level='WARNING' if args.silent else 'INFO')
    dirs = (path for path in args.files if path.is_dir())
    inner_files = (
        path for dir in dirs for path in dir.iterdir()
        if path.is_file() and path.suffix is '.mol'
    )
    args_files = (
        path for path in args.files
        if path.is_file() and path.suffix is '.mol'
    )
    for file in chain(args_files, inner_files):
        fix_file(file)


if __name__ == '__main__':
    main()



import logging
import pathlib
import threading
import tkinter as tk
import tkinter.ttk as ttk
from tkinter.filedialog import (askopenfilename, askopenfilenames,
                                askdirectory)

from backuper import Backuper


class SourceFrame(tk.Frame):
    def __init__(self, master):
        super().__init__(master)
        self.master = master
        view_frame = tk.LabelFrame(
            self, text='Source files and directories:'
        )
        view_frame.grid(row=0, column=0, sticky='nwes')
        self.tree = ttk.Treeview(view_frame)
        self.tree.grid(row=0, column=0, sticky='nwse')
        self.tree['show'] = 'tree'
        source_bar = ttk.Scrollbar(
            view_frame, orient='vertical', command=self.tree.yview
        )
        source_bar.grid(row=0, column=1, sticky='nse')
        buttons_frame = tk.Frame(self)
        buttons_frame.grid(row=0, column=1, pady=7, sticky='nwse')
        make_button(
            buttons_frame, 'Add Files', 0, 1, command=self.add_file
        )
        make_button(
            buttons_frame, 'Add Folder', 1, 1, command=self.add_dir
        )
        make_button(
            buttons_frame, 'Add Folder Tree', 2, 1, command=self.add_tree
        )
        make_button(
            buttons_frame, 'Remove Selected', 3, 1, command=self.remove_item
        )
        tk.Grid.columnconfigure(self, 0, weight=1)
        tk.Grid.rowconfigure(self, 0, weight=1)
        tk.Grid.columnconfigure(view_frame, 0, weight=1)
        tk.Grid.rowconfigure(view_frame, 0, weight=1)

    def add_file(self):
        paths = askopenfilenames()
        for path in paths:
            if str(pathlib.Path(path).resolve()) \
                    not in self.master.backuper.config['SOURCE']:
                path = self.master.backuper.add_source(path, 'f')
                self.tree.insert('', 'end', text=str(path))

    def add_dir(self):
        path = askdirectory()
        if path and str(pathlib.Path(path).resolve()) \
                not in self.master.backuper.config['SOURCE']:
            path = self.master.backuper.add_source(path, 'd')
            self.tree.insert('', 'end', text=str(path) + '\\')

    def add_tree(self):
        path = askdirectory()
        if path and str(pathlib.Path(path).resolve()) \
                not in self.master.backuper.config['SOURCE']:
            path = self.master.backuper.add_source(path, 'r')
            self.tree.insert('', 'end', text=str(path) + '\\*')

    def remove_item(self):
        for item in self.tree.selection():
            path = str(
                pathlib.Path(self.tree.item(item)['text'].strip('*')).resolve()
            )
            done = self.master.backuper.config.remove_option('SOURCE', path)
            if done:
                self.tree.delete(item)


class IgnoredFrame(tk.Frame):
    def __init__(self, master):
        super().__init__(master)
        self.master = master
        label_frame = tk.LabelFrame(
            self, text='Ignored files and directories:'
        )
        label_frame.grid(row=0, column=0, sticky='nwes')
        self.tree = ttk.Treeview(label_frame)
        self.tree['show'] = 'tree'
        self.tree.grid(row=0, column=0, sticky='nwse')
        self.tree.config(height=5)
        source_bar = ttk.Scrollbar(
            label_frame, orient='vertical', command=self.tree.yview
        )
        source_bar.grid(row=0, column=1, sticky='nse')
        buttons_frame = tk.Frame(self)
        buttons_frame.grid(row=0, column=1, pady=7, sticky='nwes')
        make_button(
            buttons_frame, 'Add Files', 0, 1, command=self.add_ignored_file
        )
        make_button(
            buttons_frame, 'Add Folder', 1, 1, command=self.add_ignored_dir
        )
        make_button(
            buttons_frame, 'Remove Selected', 2, 1,
            command=self.remove_item
        )
        tk.Grid.columnconfigure(self, 0, weight=1)
        tk.Grid.columnconfigure(self, 0, weight=1)
        tk.Grid.columnconfigure(label_frame, 0, weight=1)
        tk.Grid.columnconfigure(label_frame, 0, weight=1)

    def add_ignored_file(self):
        paths = askopenfilenames()
        for path in paths:
            if str(pathlib.Path(path).resolve()) \
                    not in self.master.backuper.ignored:
                path = self.master.backuper.add_ignored(path)
                self.tree.insert('', 'end', text=str(path))

    def add_ignored_dir(self):
        path = askdirectory()
        if path and str(pathlib.Path(path).resolve()) \
                not in self.master.backuper.ignored:
            path = self.master.backuper.add_ignored(path)
            self.tree.insert('', 'end', text=str(path) + '\\')

    def remove_item(self):
        for item in self.tree.selection():
            path = str(
                pathlib.Path(self.tree.item(item)['text'].strip('*')).resolve()
            )
            done = self.master.backuper.config.remove_option('IGNORE', path)
            if done:
                self.tree.delete(item)


class App(tk.Tk):
    def __init__(self):
        super().__init__()
        self.title('zeetoo backuper')
        tk.Grid.columnconfigure(self, 0, weight=1)
        tk.Grid.rowconfigure(self, 1, weight=1)
        dest_frame = tk.Frame(self)
        dest_frame.grid(row=0, column=0, sticky='nwe')
        tk.Label(dest_frame, text='Destination').grid(row=0, column=0,
                                                      sticky='w')
        self.dest_var = tk.StringVar()
        dest_entry = tk.Entry(dest_frame, textvariable=self.dest_var)
        dest_entry.grid(row=0, column=1, sticky='we')
        dest_entry.bind('<FocusOut>', lambda e: self.changed_dest())
        choose = make_button(
            dest_frame, 'Choose', 0, 2, 'we', command=self.choose_dest
        )
        tk.Grid.columnconfigure(dest_frame, 1, weight=1)
        tk.Grid.rowconfigure(dest_frame, 0, weight=1)

        self.source = SourceFrame(self)
        self.source.grid(row=1, column=0, sticky='nwes')

        self.ignored = IgnoredFrame(self)
        self.ignored.grid(row=2, column=0, sticky='nwe')

        time_frame = tk.Frame(self)
        time_frame.grid(row=3, column=0, sticky='nwes')
        tk.Label(time_frame, text='Backup frequency: ').grid(row=0, column=0)
        self.period_var = tk.StringVar()
        period_box = ttk.Combobox(time_frame, textvariable=self.period_var)
        period_box['values'] = ['DAILY', 'WEEKLY', 'MONTHLY']
        period_box.grid(row=0, column=1)
        period_box.config(width=11)
        period_box.bind(
            '<<ComboboxSelected>>', lambda e: self.changed_time()
        )
        tk.Label(time_frame, text='Hour: ').grid(row=0, column=2)
        self.hour_var = tk.StringVar()
        hour_box = ttk.Combobox(time_frame, textvariable=self.hour_var)
        hour_box['values'] = [f'{x:0>2}' for x in range(1, 25)]
        hour_box.grid(row=0, column=3)
        hour_box.config(width=3)
        hour_box.bind(
            '<<ComboboxSelected>>', lambda e: self.changed_time()
        )
        tk.Label(time_frame, text='Min: ').grid(row=0, column=4)
        self.min_var = tk.StringVar()
        min_box = ttk.Combobox(time_frame, textvariable=self.min_var)
        min_box['values'] = [f'{x:0>2}' for x in range(0, 60, 5)]
        min_box.grid(row=0, column=5)
        min_box.config(width=3)
        min_box.bind(
            '<<ComboboxSelected>>', lambda e: self.changed_time()
        )
        bottom_frame = tk.Frame(self)
        bottom_frame.grid(row=4, column=0, sticky='nwes')
        sched_button = make_button(
            bottom_frame, 'Schedule', 0, 0, command=self.schedule
        )
        unsched_button = make_button(
            bottom_frame, 'Unschedule', 0, 1, command=self.unschedule
        )
        self.run_text_var = tk.StringVar()
        self.run_text_var.set('Run Now')
        self.run_button = make_button(
            bottom_frame, '', 0, 2, textvariable=self.run_text_var,
            command=self.run_now
        )
        tk.Frame(bottom_frame).grid(row=0, column=3, sticky='we')
        load_button = make_button(
            bottom_frame, 'Load Config', 0, 4, command=self.load
        )
        self.save_button = make_button(
            bottom_frame, 'Save Config', 0, 5, command=self.save
        )
        tk.Grid.columnconfigure(bottom_frame, 3, weight=1)
        tk.Grid.rowconfigure(bottom_frame, 0, weight=1)

        self.HOME_DIR = pathlib.Path.home().joinpath(
            'AppData', 'Local', 'zeetoo', 'backuper'
        )
        self.HOME_DIR.mkdir(parents=True, exist_ok=True)
        self.backuper = Backuper(
            self.HOME_DIR / 'config.ini'
        )
        self.fill_gui()

    def choose_dest(self):
        path = askdirectory()
        if path:
            self.backuper.destination = path
            self.dest_var.set(self.backuper.destination)

    def changed_dest(self):
        path = self.dest_var.get()
        self.backuper.destination = path

    def changed_time(self):
        self.backuper.set_time(
            self.period_var.get(),
            int(self.hour_var.get()),
            int(self.min_var.get())
        )

    def save(self):
        self.backuper.save_config()
        self.save_button.configure(text='Saved!')
        thread = threading.Timer(
            2, self.save_button.configure, kwargs={'text': 'Save Config'}
        )
        thread.start()

    def load(self):
        file = askopenfilename(
            defaultextension='ini', initialfile='config.ini',
            filetypes=[('Config files', '*.ini'), ('All files', '*.*')]
        )
        if file:
            self.backuper.load_config(file)
            self.fill_gui()

    def schedule(self):
        self.backuper.schedule()

    def unschedule(self):
        self.backuper.unschedule()

    def _run(self):
        self.run_text_var.set('Running...')
        self.run_button.config(state='disabled')
        self.backuper.backup()
        try:
            self.run_text_var.set('Run Now')
            self.run_button.config(state='normal')
        except Exception:
            logging.debug('Could not change button state')

    def run_now(self):
        thread = threading.Thread(target=self._run, args=[self.backuper])
        thread.start()

    def fill_gui(self):
        self.dest_var.set(self.backuper.destination)
        self.source.tree.delete(*self.source.tree.get_children())
        self.ignored.tree.delete(*self.ignored.tree.get_children())
        for path, mode in self.backuper.config['SOURCE'].items():
            appendix = '\\' if mode == 'd' else '\\*' if mode == 'r' else ''
            self.source.tree.insert('', 'end', text=path + appendix)
        for path in self.backuper.ignored:
            appendix = '\\' if pathlib.Path(path).is_dir() else ''
            self.ignored.tree.insert('', 'end', text=path + appendix)
        self.period_var.set(self.backuper.config['BACKUP']['schedule'])
        hour, minute = self.backuper.config['BACKUP']['starttime'].split(':')
        self.hour_var.set(hour)
        self.min_var.set(minute)


def make_button(master, text, row=0, column=0, sticky='nwe', textvariable=None,
                command=None):
    b = ttk.Button(
        master, text=text, textvariable=textvariable, command=command
    )
    b.grid(row=row, column=column, sticky=sticky)
    b.config(width=15)
    return b


def main(argv=None):
    logging.basicConfig(
        format='%(levelname)s: %(message)s', level=logging.INFO
    )
    root = App()
    root.mainloop()


if __name__ == '__main__':

    main()



import argparse
import configparser
import datetime
import logging
import pathlib
import shutil
import subprocess
import sys


class Backuper:

    def __init__(self, configfile: str = '') -> None:
        self.configfile = configfile if configfile else \
            f'config_{int(datetime.datetime.now().timestamp())}.ini'
        if configfile and self.configfile.exists():
            self.config = configparser.ConfigParser(
                allow_no_value=True, delimiters=('=',)
            )
            with open(self.configfile, 'r') as file:
                self.config.read_file(file)
        if configfile and not self.configfile.exists():
            logging.warning('Specified file does not exist. '
                            'Initializing with standard parameters')
        # raise FileNotFoundError(f'No such file: {configfile}')
        if not configfile or not self.configfile.exists():
            self.config = configparser.ConfigParser(
                allow_no_value=True, delimiters=('=',)
            )
            self.config['BACKUP'] = {'destination': str(pathlib.Path.cwd()),
                                     'taskname': 'zeetoo backup',
                                     'schedule': 'DAILY',
                                     'starttime': '03:00'}
            self.config['SOURCE'] = {}
            self.config['IGNORE'] = {}
        self._copyists = {
            'f': self.copy_file,
            'd': self.copy_directory,
            'r': self.copy_recursive
        }

    @property
    def configfile(self) -> pathlib.Path:
        return self.__configfile

    @configfile.setter
    def configfile(self, path) -> None:
        self.__configfile = pathlib.Path(path).resolve()

    def save_config(self, file: str = '') -> None:
        if file:
            self.configfile = file
        with open(self.configfile, 'w') as configfile:
            self.config.write(configfile)
        logging.debug(f'Config saved to {self.configfile}')

    def load_config(self, file: str = '') -> None:
        file = file if file else self.configfile
        config = configparser.ConfigParser(
            allow_no_value=True, delimiters=('=',)
        )
        with open(file, 'r') as configfile:
            config.read_file(configfile)
        self.configfile = file
        self.config = config
        logging.debug(f'Config loaded from {file}')

    @property
    def destination(self) -> str:
        return pathlib.Path(self.config['BACKUP']['destination'])

    @destination.setter
    def destination(self, destination: str) -> None:
        path = pathlib.Path(destination)
        if not path.exists():
            path.mkdir(parents=True)
            logging.debug("Created backup destination directory.")
        self.config['BACKUP']['destination'] = str(path.resolve())
        logging.debug(f'Destination set to {destination}')

    @property
    def sources(self) -> iter:
        return (
            (pathlib.Path(path), mode) for path, mode
            in self.config['SOURCE'].items()
            if path not in self.ignored
        )

    @property
    def ignored(self) -> set:
        return set(self.config['IGNORE'])

    def add_source(self, source: str, mode: str) -> pathlib.Path:
        if mode not in ('f', 'd', 'r'):
            raise ValueError(
                "Invalid mode. Mode should be one of: 'f', 'd', 'r'"
            )
        path = pathlib.Path(source).resolve()
        if not path.exists():
            logging.warning(f"Source doesn't exist: {source}")
        # raise if source incompatible with mode
        self.config['SOURCE'][str(path)] = mode
        return path

    def remove_source(self, source: str) -> bool:
        return self.config.remove_option('SOURCE', source)

    def add_ignored(self, ignored: str) -> pathlib.Path:
        path = pathlib.Path(ignored).resolve()
        self.config['IGNORE'][str(path)] = None
        logging.debug(f"Ignored path registered: {ignored}")
        return path

    def remove_ignored(self, ignored: str) -> bool:
        return self.config.remove_option('IGNORED', ignored)

    def count_files(self, directory: pathlib.Path = None) -> int:
        ignored = self.ignored
        if directory is None:
            files = len([path for path, mode in self.sources if path.is_file()])
            dirs = (
                (path, mode) for path, mode in self.sources if path.is_dir()
            )
            for directory, mode in dirs:
                if mode == 'd':
                    files += len([
                        path for path in directory.iterdir() if path.is_file()
                        and str(path) not in ignored
                    ])
                elif mode == 'r':
                    files += self.count_files(directory)
                else:
                    raise ValueError(f"Invalid mode for directory {mode}")
        else:
            files = len([path for path in directory.iterdir() if path.is_file()
                         and str(path) not in self.ignored])
            dirs = (path for path in directory.iterdir() if path.is_dir()
                    and str(path) not in self.ignored)
            for path in dirs:
                files += self.count_files(path)
        return files

    def backup(self) -> None:
        basedest = pathlib.Path(self.config['BACKUP']['destination'])
        msg = f'Starting backup using {self.configfile.name} specification.' \
              if self.configfile.exists() else 'Starting backup as specified ' \
                                               'internally.'
        logging.info(msg)
        for path, mode in self.sources:
            if not path.exists():
                logging.warning(f"Specified source not found: {path}")
                continue
            if mode == 'f':
                logging.info(f'Moving to next source: {path}')
                dest = pathlib.Path(basedest, path.parent.name, path.name)
                if not dest.parent.exists():
                    dest.parent.mkdir(parents=True)
                    logging.debug(f"Dir created: {dest.parent}")
            else:
                dest = pathlib.Path(basedest, path.name)
            copyist = self._copyists[mode]
            copyist(path, dest)
        logging.info('Backup done.')

    def copy_file(self, src: pathlib.Path, dest: pathlib.Path) -> None:
        if not dest.exists() or src.stat().st_mtime > dest.stat().st_mtime:
            # copy
            shutil.copy2(src, dest)
            logging.debug(f"Copied file: {src}")
        elif dest.exists() and src.stat().st_mtime < dest.stat().st_mtime:
            # rename and copy
            filetime = datetime.datetime.fromtimestamp(dest.stat().st_mtime)
            filetime = filetime.strftime('_newer_%Y-%m-%d_%H-%M')
            newer_name = dest.name.split('.')
            newer_name[0] += filetime
            newer_name = '.'.join(newer_name)
            newer_path = pathlib.Path(dest.parent, newer_name)
            dest.rename(newer_path)
            shutil.copy2(src, dest)
            logging.warning(
                f"Newer file version found in backup directory:\n\t{src}"
                f"\n\tNewer file renamed to {newer_name}"
            )
        else:
            # leave it be
            logging.debug(f"File didn't change: {src}")
            pass

    def copy_directory(self, src: pathlib.Path, dest: pathlib.Path) -> None:
        logging.info(f'Moving to next source: {src}')
        if not dest.exists():
            dest.mkdir(parents=True)
            logging.debug(f"Dir created: {dest}")
        files = (
            path for path in src.iterdir()
            if path.is_file() and not str(path) in self.ignored
        )
        for path in files:
            self.copy_file(path, pathlib.Path(dest, path.name))

    def copy_recursive(self, src: pathlib.Path, dest: pathlib.Path) -> None:
        # log 'debug: copying recursively {dir}'
        self.copy_directory(src, dest)
        dirs = (
            path for path in src.iterdir()
            if path.is_dir() and not str(path) in self.ignored
        )
        for dir_ in dirs:
            self.copy_recursive(dir_, pathlib.Path(dest, dir_.name))

    def set_time(
            self, period: str = None, hour: int = None, minute: int = None
    ) -> None:
        if period:
            self.config['BACKUP']['schedule'] = period
        if hour is None and minute is None:
            return
        curr_hour, curr_minute = self.config['BACKUP']['starttime'].split(':')
        hour = hour if hour is not None else int(curr_hour)
        minute = minute if minute is not None else int(curr_minute)
        self.config['BACKUP']['starttime'] = f"{hour:0>2}:{minute:0>2}"

    @property
    def task_name(self) -> str:
        if 'taskname' not in self.config['BACKUP']:
            raise ValueError('Task name not specified.')
        else:
            return self.config['BACKUP']['taskname']

    @task_name.setter
    def task_name(self, name: str) -> None:
        self.config['BACKUP']['taskname'] = name

    @property
    def schtasks_command(self) -> list:
        script = pathlib.Path(__file__).resolve()
        cmd = [
            'schtasks', '/create', '/F', '/TN', self.task_name,
            '/SC', self.config['BACKUP']['schedule'],
            '/ST', self.config['BACKUP']['starttime'], '/TR',
            f'"{sys.executable}" "{script}" -c "{self.configfile.resolve()}" -b'
        ]
        return cmd

    def schedule(self) -> None:
        self.save_config()
        cmd = self.schtasks_command
        subprocess.run(cmd, check=True)

    def unschedule(self) -> None:
        cmd = ['schtasks', '/delete', '/TN', f"{self.task_name}", '/F']
        subprocess.run(cmd, check=True)


def get_parser():
    parser = argparse.ArgumentParser(prog='zeetoo backuper')
    parser.add_argument(
        '--configfile', '-c', metavar='path', default='',
        help="path to Backupers' config.ini file"
    )
    parser.add_argument(
        '--destination', '-d', metavar='path',
        help="path to directory, in which copied files will be stored"
    )
    parser.add_argument(
        '--add-sources', '-a', metavar='path', nargs='*', dest='add',
        help="directories to backup"
    )
    parser.add_argument(
        '--recursively', '-r', metavar='path', nargs='*',
        help="directories to backup, including all it's subdirectories"
    )
    parser.add_argument(
        '--ignore', '-i', metavar='path', nargs='*',
        help="files and folders ignored"
    )
    parser.add_argument(
        '--taskname', '-n',
        help='name of the task scheduled, defaults to "zeetoo backup"'
    )
    parser.add_argument(
        '--period', '-p', choices=['once', 'daily', 'weekly', 'monthly'],
        help='how often should backup be carried out'
    )
    parser.add_argument(
        '--hour', '-H', type=int, choices=range(24), metavar='H',
        help='at which hour should task start, should be an integer between '
             '0 and 23'
    )
    parser.add_argument(
        '--minute', '-m', type=int, choices=range(60), metavar='M',
        help='how many minutes after specified hour should task start, '
             'should be an integer between 0 and 59'
    )
    parser.add_argument(
        '--schedule', '-s', action='store_true',
        help='schedules backup specified in config.ini file'
    )
    parser.add_argument(
        '--unschedule', '-u', action='store_true',
        help='removes backup task of specified task name from schedule'
    )
    parser.add_argument(
        '--run-backup', '-b', action='store_true', dest='run',
        help='runs backup specified in config.ini file'
    )
    parser.add_argument(
        '--debug', '-D', action='store_true',
        help='prints schtasks_command generated from config.ini file'
    )
    parser.add_argument(
        '--verbose', '-v', action='store_true', help='shows additional messages'
    )
    parser.add_argument(
        '--version', '-V', action='version', version='%(prog)s 0.1'
    )
    return parser


def main(argv=None):
    args = get_parser().parse_args(argv)
    if args.verbose:
        logging.getLogger().setLevel(logging.INFO)
    if args.debug:
        logging.getLogger().setLevel(logging.DEBUG)
    backuper = Backuper(args.configfile)
    if args.taskname:
        backuper.task_name = args.taskname
    if args.period or args.hour or args.minute:
        backuper.set_time(args.period, args.hour, args.minute)
    if args.destination:
        backuper.destination = args.destination
    if args.add:
        for arg in args.add:
            backuper.add_source(arg, 'd')
    if args.recursively:
        for arg in args.recursively:
            backuper.add_source(arg, 'r')
    if args.ignore:
        for arg in args.ignore:
            backuper.add_ignored(arg)
    if args.schedule:
        backuper.schedule()
    if args.unschedule:
        backuper.unschedule()
    if args.run:
        backuper.backup()


if __name__ == '__main__':
    main()



import argparse
import logging as lgg
import re
import pathlib


coords = re.compile(r'\s*(-?\d+\.\d+)\s+(-?\d+\.\d+)\s+(-?\d+\.\d+)\s*(\w\w?)')


def get_parser(argv=None):
    prsr = argparse.ArgumentParser(
        description="Generate .gjf files from .sdf file."
    )
    prsr.add_argument('sdf', type=pathlib.Path, help="source .sdf file")
    prsr.add_argument(
        '-o', '--out_dir', type=pathlib.Path, help="Path to output directory."
    )
    prsr.add_argument(
        '-r', '--route', required=True, help="Calculations specification."
    )
    prsr.add_argument(
        '-d', '--dscr', default='', help="Brief description of the calculation."
    )
    prsr.add_argument(
        '-l', '--link', default='', help="space separated link-0 commands."
    )
    prsr.add_argument(
        '-s', '--sufix', default='',
        help="Everything after molecule specification."
    )
    prsr.add_argument(
        '-c', '--charge', required=True, type=int, help="Molecule charge."
    )
    prsr.add_argument(
        '-m', '--multiplicity', required=True, type=int,
        help="Molecule spin multiplicity."
    )
    prsr.add_argument(
        '-n', '--name',
        help='Core of output files names, it will be appended with [number].gjf'
    )
    prsr.add_argument(
        '-f', '--first_num', type=int, default=0,
        help='Number of first conformer.'
    )
    return prsr
    
    
def get_coords(file, line):
    match = coords.search(line)
    while match is not None:
        x, y, z, a = match.groups()
        yield a, float(x), float(y), float(z)
        match = coords.search(file.readline())


def get_molecules(source: pathlib.Path):
    with source.open() as sdf:
        for line in sdf:
            if coords.search(line) is not None:
                yield get_coords(sdf, line)
        
        
def save_molecule(
    dest: pathlib.Path, charge: int, multipl: int, coords: list, command: str,
    comment: str = '', prefix: str = '', sufix: str = ''
):
    with dest.open('w') as gjf:
        if prefix:
            gjf.write(prefix)
            gjf.write('\n')
        if not command.startswith('#'):
            command = '# ' + command
        gjf.write(command)
        gjf.write('\n\n')
        if comment:
            gjf.write(comment)
            gjf.write('\n\n')
        gjf.write('0 1\n')
        for a, x, y, z in coords:
            gjf.write(
                f" {a: <2} {x: > .7f} {y: > .7f} {z: > .7f}\n"
            )
        if sufix:
            gjf.write('\n')
            gjf.write(sufix)
        gjf.write('\n\n')
        

def main(argv: list = None):
    args = get_parser().parse_args(argv)
    args.out_dir.mkdir(exist_ok=True)
    gjfname = args.name if args.name is not None else args.sdf.stem
    out_dir = args.out_dir if args.out_dir is not None else args.sdf.parent
    for num, mol in enumerate(get_molecules(args.sdf)):
        output_file = out_dir / (gjfname + f'{num+args.first_num:0>3}.gjf')
        save_molecule(
            dest=output_file,
            charge=args.charge,
            multipl=args.multiplicity,
            coords=mol,
            command=args.route,
            comment=args.dscr,
            prefix=args.link,
            sufix=args.sufix
        )


if __name__ == '__main__':
    
    main()



import argparse
import importlib
import pathlib
import sys


def get_parser(available_modules):
    prs = argparse.ArgumentParser(
        prog='zeetoo',
        description=f'A collection of various python scripts for use in '
                    f'Team II IOC PAS. Modules available: '
                    f'{", ".join(available_modules)}. For information '
                    f'regarding specific module run "%(prog)s MODULE -h".',
        usage='%(prog)s [-h] MODULE [options_for_module]'
    )
    prs.add_argument(
        'module', choices=available_modules, metavar='MODULE',
        help='Module to run.'
    )
    return prs


def main():
    directory = pathlib.Path(__file__).resolve().parent
    modules = [
        f.stem for f in directory.glob('*.py') if not f.stem.startswith('_')
    ]
    validator = get_parser(modules)
    _, module, *other = sys.argv
    args = validator.parse_args([module])
    module = args.module
    module = importlib.import_module(module)
    module.main(other)


if __name__ == '__main__':

    main()



import argparse
import logging
import olefile
import pathlib
import zipfile
try:
    from xml.etree.cElementTree import XML
except ImportError:
    from xml.etree.ElementTree import XML

REL = '{http://schemas.openxmlformats.org/package/2006/relationships}Relationship'
OLEREL = 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/oleObject'
RELID = '{http://schemas.openxmlformats.org/officeDocument/2006/relationships}id'
OLEOBJ = '{urn:schemas-microsoft-com:office:office}OLEObject'
WORD_NAMESPACE = '{http://schemas.openxmlformats.org/wordprocessingml/2006/main}'
PAR = WORD_NAMESPACE + 'p'
TXT = WORD_NAMESPACE + 't'


def get_ole_rels(ziphandle):
    """Returns a dictionary of embedded objects Ids (keys) and locations
    (values)."""
    tree = XML(ziphandle.read("word/_rels/document.xml.rels"))
    rels = {
        a['Id']: a['Target'] for a in
        (e.attrib for e in tree.iter(REL))
        if a['Type'] == OLEREL
    }
    return rels


def find_embeddings(ziphandle, title_first=False):
    """Yields two-element tuple with OLEObject element and paragraph element
    if OLEObject found is a ChemDraw file."""
    tree = XML(ziphandle.read('word/document.xml'))
    embedding, paragrph = None, None
    for elem in tree.iter():
        # yield only if object embedded is ChemDraw file
        if elem.tag == OLEOBJ and elem.attrib['ProgID'].startswith('ChemDraw'):
            embedding = elem
            if title_first:
                # paragraph should be already known so yield and reset
                yield (embedding, paragrph)
                embedding = None
        elif elem.tag == PAR:
            paragrph = elem
            if embedding is not None and not title_first:
                # yield and reset only if embedding found
                yield (embedding, paragrph)
                embedding = None
        else:
            continue


def extract_embeddings(
        path: pathlib.Path, out: pathlib.Path, words_in_name: int = 5,
        title_first: bool = False
             ):
    """Opens a .docx file given in `path`, reads all embedded ChemDraw
    objects and writes them to `out` directory, using first `words_in_name`
    words of the first paragraph after object (or the one directly before,
    if `title_first` is True) as the file name."""
    with zipfile.ZipFile(path) as ziphandle:
        rels = get_ole_rels(ziphandle)
        for embedding, paragraph in find_embeddings(ziphandle, title_first):
            embedded_file = 'word/' + rels[embedding.attrib[RELID]]
            text = [node.text for node in paragraph.iter(TXT) if node.text]
            text = ''.join(text)
            title = ' '.join(text.split()[:words_in_name])
            ole = ziphandle.read(embedded_file)
            with olefile.OleFileIO(ole) as ole_handle:
                content = ole_handle.openstream('CONTENTS').read()
            outfile = out / (title + '.cdx')
            with outfile.open('wb') as out_handle:
                out_handle.write(content)


def get_parser():
    prs = argparse.ArgumentParser('Extract .cdx files from a .docx file.')
    prs.add_argument(
        'file', type=pathlib.Path, help='.docx file with .cdx files embedded.'
    )
    prs.add_argument(
        '-o', '--output-dir', type=pathlib.Path,
        help='Output directory. Defaults to [file-location]/[file-name]-cdx/'
    )
    prs.add_argument(
        '-w', '--words', type=int, default=5,
        help='Number of words included in name of .cdx file.'
    )
    prs.add_argument(
        '-p', '--preceding-title', action='store_true',
        help='Look for scheme description before the scheme.'
    )
    return prs


def main(argv=None):
    logging.basicConfig(level='DEBUG')
    prs = get_parser()
    args = prs.parse_args(argv)
    outdir = args.output_dir or args.file.parent / (args.file.stem + '-cdx')
    outdir.mkdir(exist_ok=True)
    extract_embeddings(args.file, outdir, args.words, args.preceding_title)


if __name__ == "__main__":
    main()



import argparse
import os
import logging as lgg
from math import floor, sqrt
import rdkit
from rdkit import Chem
from rdkit.Chem import AllChem


def get_args(argv=None):
    """Parses given arguments and returns argparse.Namespace object."""
    prsr = argparse.ArgumentParser(
        description="Perform a conformational search on given molecules."
    )
    group = prsr.add_mutually_exclusive_group(required=True)
    group.add_argument(
        '-m', '--molecules', nargs='+',
        help='One or more files with molecule specification.'
    )
    group.add_argument(
        '-d', '--directory', help='Directory with .mol files.'
    )
    prsr.add_argument(
        '-o', '--output_dir', default='.\\confsearch', help='Output directory.'
    )
    prsr.add_argument(
        '-n', '--num_confs', type=int, default=10,
        help='Number of cnformers to generate.'
    )
    prsr.add_argument(
        '-r', '--rms_tresh', type=float, default=1,
        help='Maximum RMSD of conformers.'
    )
    prsr.add_argument(
        '-e', '--energy_window', type=float, default=5,
        help='Maximum energy difference from lowest-energy conformer '
        'in kcal/mol.'
    )
    prsr.add_argument(
        '-c', '--max_cycles', type=int, default=10,
        help='Maximum number of energy minimization cycles.'
    )
    prsr.add_argument(
        '-f', '--fixed', type=int, nargs='+', default=(),
        help='Indices (starting at 1) of atoms fixed during molecule embedding.'
    )
    prsr.add_argument(
        '-x', '--constraints',
        help='File with constraints specified in format '
             '"kind a [b [c [d]] rel min] max [const]", one for line. `kind` '
             'should be one of: P (position), D (distance), A (angle), T '
             '(torsion). Number of required atoms indices depends on `kind` '
             'given and should be 1, 2, 3 or 4 respectively. Atoms indices '
             'start at 1. `rel` should be 0 or 1 and specifies if `min` and '
             '`max` values should be treated as absolute values or relative '
             'to current value. `min` and `max` should be floats, representing '
             'minimum and maximum value of constrained property in relevant '
             'units (angstroms or degrees). `rel` and `min` should be omitted '
             'if `kind` is P. `const` is force constant for given constraint, '
             'should be integer or float, defaults to 1e5.'
    )
    prsr.add_argument(
        '-V', '--verbose', action='store_true',
        help='Sets logging level to INFO.'
    )
    prsr.add_argument(
        '-D', '--debug', action='store_true',
        help='Sets logging level to DEBUG.'
    )
    return prsr.parse_args(argv)


def parse_constraints(params, std_const=1e5):
    """Transforms a string to list of parameters taken by appropriate method
    of ForceField object. String should be in one of the following forms:
        P x min [const]
        D x x rel min max [const]
        A x x x rel min max [const]
        T x x x x rel min max [const]
    where:
    - the first letter defines type of constraint (P = position of atom,
      D = distance between atoms, A = dihedral angle, T = torsion angle),
    - x stands for index of involved atom;
    - `rel` should be 0 or 1 and specifies if `min` and `max` values should be
      treated as absolute values or relative to current value; doesnt apply to P;
    - `min` and `max` represent minimum and maximum value of constrained property
      in relevant units (angstroms or degrees);
    - `const` is force constant for given constraint, if omitted, value of
      `std_const` is used."""
    params = params.split()
    cn = {  # number of coordinates
        'a': 3, 'p': 1, 'd': 2, 't': 4
    }
    kind = params[0].lower()
    atoms, left = params[1:cn[kind] + 1], params[cn[kind] + 1:]
    if kind == 'p':
        other = float(left[0]), (float(left[1]) if len(left) == 2 else std_const)
    else:
        other = bool(left[0]), float(left[1]), float(left[2]), \
                (float(left[3]) if len(left) == 4 else std_const)
    return (kind, *(int(a)-1 for a in atoms), *other)


def get_constraints(file, std_const=1e5):
    """Opens given file and parses it, returning list (one element for line
    in file) of lists of parameters for use in appropriate method of ForceField
    object."""
    with open(file) as f:
        return [parse_constraints(line, std_const) for line in f]


def make_constraints(ff, constraints):
    """Adds specified constraints to given ForceField object, returns that
    object."""
    cm = {  # constraints makers
        'a': ff.MMFFAddAngleConstraint, 'p': ff.MMFFAddPositionConstraint,
        'd': ff.MMFFAddDistanceConstraint, 't': ff.MMFFAddTorsionConstraint
    }
    for kind, *params in constraints:
        cm[kind](*params)
    return ff


def find_lowest_energy_conformer(
        molecule, num_confs, rms_tresh, max_cycles, coord_map, constraints
):
    """Performs a conformational search, keeping track of lowest energy
    conformer.

    Parameters
    ----------
    molecule : rdkit.Mol
        Molecule on which conformational search should be performed.
    num_confs : int
        Number of conformers to generate.
    rms_tresh : float
        Minimum RMS for conformer to be considered different.
    max_cycles : int
        Maximum number of attempts to optimize conformer.
    coord_map : dict
        A mapping of atoms, that should stay in specified positions when
        creating new conformer; key should be an atom index and value should be
        list of cartesian coordinates.
    constraints : list of lists
        List of parameters specifying constraints, if empty list given,
        optimization without any constraints is performed.

    Returns
    -------
    list of [molecule, min_id, min_en, energies], where:
        molecule is molecue with new conformer embedded;
        min_id is id of conformer of lowest energy;
        min_en is energy value of lowest energy conformer;
        energies is a mapping of {id: energy value} for each conformer generated.
    """
    ids = AllChem.EmbedMultipleConfs(
        molecule, numConfs=num_confs, pruneRmsThresh=rms_tresh,
        coordMap=coord_map
    )
    lgg.info("Conformers initialized, starting minimization.")
    min_en, min_id = float('inf'), -1
    energies = {}
    stereo = Chem.FindMolChiralCenters(molecule)
    mp = AllChem.MMFFGetMoleculeProperties(molecule)
    for cid in ids:
        if cid and cid % 100 == 0:
            lgg.info(f"Minimization progress: {cid}/{len(ids)}")
        ff = AllChem.MMFFGetMoleculeForceField(molecule, mp, confId=cid)
        ff.Initialize()
        ff = make_constraints(ff, constraints)
        for cycle in range(max_cycles):
            if not ff.Minimize():
                # ff.Minimize() returns 0 on success
                break
        else:
            molecule.RemoveConformer(cid)
            lgg.debug(f"Conf {cid} ignored: ff.Minimize() unsuccessfull")  
            continue
        energy = ff.CalcEnergy()
        lgg.debug(f"Conf {cid} lowest energy: {energy}")
        energies[cid] = energy
        if energy < min_en:
            lgg.debug(f"Conf {cid} is new min with energy {energy}")
            min_en, min_id = energy, cid
        lgg.debug(
            f"Conf {cid} energy: {energy} kcal/mol"
        )
    lgg.info(
        f"Lowest conformer found: {min_en} kcal/mol"
    )
    return molecule, min_id, min_en, energies
    

def rms_sieve(molecule, energies, threshold):
    """Filters similar conformers after optimization, based on threshold given.
    Always discards conformer of higher energy.

    Parameters
    ----------
    molecule : rdkit.Mol
        Molecule with optimized conformers embedded.
    energies : dict
        mapping of {id: energy value} for each conformer in molecule.
    threshold : float
        Minimum RMS to treat conformers as different.

    Returns
    -------
    rdkit.Mol
        Molecule with conformers filtered."""
    AllChem.AlignMolConformers(molecule)
    indices = {n: c.GetId() for n, c in enumerate(molecule.GetConformers())}
    noh = Chem.RemoveHs(molecule)
    rmsmatrix = AllChem.GetConformerRMSMatrix(noh)
    for i, rms in enumerate(rmsmatrix):
        first = floor((sqrt(8*i+1) - 1) / 2)
        second = indices[i - first * (first + 1) // 2]
        first = indices[first + 1]
        if rms > threshold:
            continue
        try:
            fen = energies[first]
            sen = energies[second]
        except KeyError:
            continue
        throwaway = first if fen > sen else second
        molecule.RemoveConformer(throwaway)
        del energies[throwaway]
        lgg.debug(f"Conf {throwaway} ignored: rms under threshold.")
    return molecule
    
    
def energy_sieve(molecule, energies, threshold):
    """Discards conformers with energies higher than lowest energy + threshold.

    Parameters
    ----------
    molecule : rdkit.Mol
        Molecule with optimized conformers embedded.
    energies : dict
        mapping of {id: energy value} for each conformer in molecule.
    threshold : float
        Maximum energy difference from lowest energy conformer.

    Returns
    -------
    rdkit.Mol
        Molecule with conformers filtered."""
    minen = min(energies.values())
    maxen = minen + threshold
    for cid, en in energies.items():
        if en > maxen:
            molecule.RemoveConformer(cid)
            lgg.debug(
                f"Conf {cid} ignored: energy {en} "
                f"higher than theshold {maxen}"
            )
    return molecule
    
    
def main(argv=None):
    """Performs a conformaional search specified by args given. Run `python
    confsearch.py --help` for details on expected arguments."""
    args = get_args(argv)
    if args.verbose:
        level = lgg.INFO
    elif args.debug:
        level = lgg.DEBUG
    else:
        level = lgg.WARNING
    lgg.basicConfig(level=level)
    lgg.info(f"Confsearch by Michał M. Więcław")
    
    if args.molecules is None:
        mol_names = [name for name in os.listdir(args.directory)
                     if name.endswith('.mol')]
        molecules = [args.directory + '\\' + name for name in mol_names]
    else:
        molecules = args.molecules
        mol_names = [mol.split('\\')[-1] for mol in molecules]
    lgg.debug(f"molecules: {molecules}")
    lgg.debug(f"molecules names: {mol_names}")

    if args.constraints:
        constraints = get_constraints(args.constraints)
    else:
        constraints = {}
    
    os.makedirs(args.output_dir, exist_ok=True)
    report_file = args.output_dir + '\\' + \
        'confsearch_report.txt'
    with open(report_file, 'w') as report:
        report.write(
            f"Confsearch -- RMSD treshold   = {args.rms_tresh} Anstrom,\n"
            f"              energy window   = {args.energy_window} kcal/mol,\n"
            f"              confs requested = {args.num_confs}\n\n"
        )
        report.write(f"Energies values of most stable conformers:\n")
        longest = max(map(len, mol_names))
        for mol, name in zip(molecules, mol_names):
            lgg.info(f"Starting with molecule {name}")
            m = Chem.MolFromMolFile(mol, removeHs=False)
            if m is not None:
                lgg.info('Molecule loaded.')
            else:
                lgg.warning(f"Couldn't load molecule {name}")
                continue
                
            atoms = {}
            for atnum in (atom.GetAtomicNum() for atom in m.GetAtoms()):
                atoms[atnum] = atoms.get(atnum, 0) + 1
            lgg.debug(f"atoms in structure: {atoms}")
            Chem.AssignStereochemistryFrom3D(m)
            lgg.debug(f"Stereochemistry found: {Chem.FindMolChiralCenters(m)}")
            
            conf = m.GetConformer()
            coord_map = {n-1: conf.GetAtomPosition(n-1) for n in args.fixed}
            
            m, cid, en, ens = find_lowest_energy_conformer(
                m, args.num_confs, args.rms_tresh, args.max_cycles, coord_map,
                constraints
            )
            num = m.GetNumConformers()
            lgg.info(f"Number of conformers optimized: {num}")
            m = energy_sieve(m, ens, args.energy_window)
            lgg.info(
                f"{num-m.GetNumConformers()} conformers outside energy window."
            )
            num = m.GetNumConformers()
            m = rms_sieve(m, ens, args.rms_tresh)
            lgg.info(
                f"{num-m.GetNumConformers()} conformers discarded by "
                "RMS sieve."
            )
            lgg.info(f"Number of conformers generated: {m.GetNumConformers()}")
            molrepr = Chem.MolToMolBlock(m, confId=cid)
            molfilename = mol.split('\\')[-1]
            outfile = args.output_dir + '\\' + molfilename.split('.')[-2] \
                + '_min_conf.mol'
            with open(outfile, 'w') as file:
                file.write(molrepr)
            lgg.info(f"Lowest energy conformer saved to {outfile}")
            report.write(
                f"{name: <{longest}} = {en: > 13.8f} kcal/mol\n"
            )
            sdfile = args.output_dir + '\\' + molfilename.split('.')[-2] \
                + '_confs.sdf'
            writer = Chem.SDWriter(sdfile)
            for conf in m.GetConformers():
                writer.write(m, confId=conf.GetId())
            writer.close()
    
    
if __name__ == '__main__':

    main()



import setuptools
from setuptools.command.install import install


class Shortcut(install):
    def run(self):
        pass


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zeetoo",
    version="0.1.0",
    author="Michał M. Więcław",
    author_email="mwieclaw@icho.edu.pl",
    license='MIT',
    description="A humble collection of various everyday utility scripts "
                "from Team II IChO PAS.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mishioo/zeetoo",
    packages=['zeetoo'],
    entry_points={
        'console_scripts': ['zeetoo=zeetoo.__main__:main']
    },
    install_requires=["openpyxl", "olefile"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
        "Development Status :: 4 - Beta"
    ],
)
